<?php

namespace Bixo\App\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\App\Forms\App as AppForm;
use Bixo\App\Http\Requests\AppResourceRequest;
use Bixo\App\Http\Resources\AppResource;
use Bixo\App\Http\Resources\AppsCollection;
use Bixo\App\Models\App;
use Bixo\App\Actions\AppAction;
use Bixo\App\Actions\AppActions;


/**
 * Resource controller class for app.
 */
class AppResourceController extends BaseController
{

    /**
     * Initialize app resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = AppForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.app.modules'), 'app', guard_url('app'));
            return $next($request);
        });
    }

    /**
     * Display a list of app.
     *
     * @return Response
     */
    public function index(AppResourceRequest $request)
    {
        $request = $request->all();
        $page = AppActions::run('paginate', $request);

        $data = new AppsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('app::app.names'))
            ->view('app::app.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display app.
     *
     * @param Request $request
     * @param Model   $app
     *
     * @return Response
     */
    public function show(AppResourceRequest $request, App $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AppResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('app::app.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('app::app.show')
            ->output();
    }

    /**
     * Show the form for creating a new app.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AppResourceRequest $request, App $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AppResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('app::app.name'))
            ->view('app::app.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new app.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AppResourceRequest $request, App $model)
    {
        try {
            $request = $request->all();
            $model = AppAction::run('store', $model, $request);
            $data = new AppResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('app::app.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/app/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/app/app'))
                ->redirect();
        }

    }

    /**
     * Show app for editing.
     *
     * @param Request $request
     * @param Model   $app
     *
     * @return Response
     */
    public function edit(AppResourceRequest $request, App $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AppResource($model);
        // return view('app::app.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('app::app.name'))
            ->view('app::app.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the app.
     *
     * @param Request $request
     * @param Model   $app
     *
     * @return Response
     */
    public function update(AppResourceRequest $request, App $model)
    {
        try {
            $request = $request->all();
            $model = AppAction::run('update', $model, $request);
            $data = new AppResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::app.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/app/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/app/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the app.
     *
     * @param Model   $app
     *
     * @return Response
     */
    public function destroy(AppResourceRequest $request, App $model)
    {
        try {

            $request = $request->all();
            $model = AppAction::run('destroy', $model, $request);
            $data = new AppResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('app::app.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/app/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/app/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
