<?php

namespace Bixo\App\Http\Controllers;

use Exception;
use Bixo\App\Actions\AppWorkflow;
use Bixo\App\Http\Requests\AppWorkflowRequest;
use Bixo\App\Http\Resources\AppResource;
use Bixo\App\Models\App;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AppWorkflowController extends BaseController
{
    /**
     * Action controller function for app.
     *
     * @param Model $app
     * @param action next action for the app.
     *
     * @return Response
     */
    public function __invoke(AppWorkflowRequest $request, App $app, $trasition)
    {
        try {
            $request = $request->all();
            $app = AppWorkflow::run($trasition, $app, $request);
            $data = new AppResource($app);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::app.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/app/' . $app->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/app/' . $app->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the app and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $app
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $app, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $app = App::findBySignedId($app);
        $transitions = $this->transitions($app, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('app::app.name') . ' ' . trans('Approvals'))
            ->view('app::public.app.workflow')
            ->layout('mobile')
            ->data(compact('app', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the app.
     *
     * @param PublicRequest $request
     * @param string $app
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $app, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $app = App::findBySignedId($app);
        $user = user();
        $app = AppWorkflow::run($transition, $app, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($app, $user)
    {
        $transitions = [];
        foreach ($app->workflow()->transitions($app) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.app.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'app' => $app->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $app->workflow()->form($value);
            $array['label'] = trans('app::app.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
