<?php

namespace Bixo\App\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\App\Forms\Key as KeyForm;
use Bixo\App\Http\Requests\KeyResourceRequest;
use Bixo\App\Http\Resources\KeyResource;
use Bixo\App\Http\Resources\KeysCollection;
use Bixo\App\Models\Key;
use Bixo\App\Actions\KeyAction;
use Bixo\App\Actions\KeyActions;


/**
 * Resource controller class for key.
 */
class KeyResourceController extends BaseController
{

    /**
     * Initialize key resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = KeyForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.app.modules'), 'app', guard_url('app'));
            return $next($request);
        });
    }

    /**
     * Display a list of key.
     *
     * @return Response
     */
    public function index(KeyResourceRequest $request)
    {
        $request = $request->all();
        $page = KeyActions::run('paginate', $request);

        $data = new KeysCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('app::key.names'))
            ->view('app::key.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display key.
     *
     * @param Request $request
     * @param Model   $key
     *
     * @return Response
     */
    public function show(KeyResourceRequest $request, Key $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new KeyResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('app::key.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('app::key.show')
            ->output();
    }

    /**
     * Show the form for creating a new key.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(KeyResourceRequest $request, Key $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new KeyResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('app::key.name'))
            ->view('app::key.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new key.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(KeyResourceRequest $request, Key $model)
    {
        try {
            $request = $request->all();
            $model = KeyAction::run('store', $model, $request);
            $data = new KeyResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('app::key.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/key/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/app/key'))
                ->redirect();
        }

    }

    /**
     * Show key for editing.
     *
     * @param Request $request
     * @param Model   $key
     *
     * @return Response
     */
    public function edit(KeyResourceRequest $request, Key $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new KeyResource($model);
        // return view('app::key.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('app::key.name'))
            ->view('app::key.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the key.
     *
     * @param Request $request
     * @param Model   $key
     *
     * @return Response
     */
    public function update(KeyResourceRequest $request, Key $model)
    {
        try {
            $request = $request->all();
            $model = KeyAction::run('update', $model, $request);
            $data = new KeyResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::key.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/key/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/key/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the key.
     *
     * @param Model   $key
     *
     * @return Response
     */
    public function destroy(KeyResourceRequest $request, Key $model)
    {
        try {

            $request = $request->all();
            $model = KeyAction::run('destroy', $model, $request);
            $data = new KeyResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('app::key.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/key/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/key/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
