<?php

namespace Bixo\App\Http\Controllers;

use Exception;
use Bixo\App\Actions\KeyWorkflow;
use Bixo\App\Http\Requests\KeyWorkflowRequest;
use Bixo\App\Http\Resources\KeyResource;
use Bixo\App\Models\Key;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class KeyWorkflowController extends BaseController
{
    /**
     * Action controller function for key.
     *
     * @param Model $key
     * @param action next action for the key.
     *
     * @return Response
     */
    public function __invoke(KeyWorkflowRequest $request, Key $key, $trasition)
    {
        try {
            $request = $request->all();
            $key = KeyWorkflow::run($trasition, $key, $request);
            $data = new KeyResource($key);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::key.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/key/' . $key->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/key/' . $key->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the key and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $key
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $key, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $key = Key::findBySignedId($key);
        $transitions = $this->transitions($key, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('app::key.name') . ' ' . trans('Approvals'))
            ->view('app::public.key.workflow')
            ->layout('mobile')
            ->data(compact('key', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the key.
     *
     * @param PublicRequest $request
     * @param string $key
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $key, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $key = Key::findBySignedId($key);
        $user = user();
        $key = KeyWorkflow::run($transition, $key, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($key, $user)
    {
        $transitions = [];
        foreach ($key->workflow()->transitions($key) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.key.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'key' => $key->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $key->workflow()->form($value);
            $array['label'] = trans('app::key.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
